"""GTK UI
"""

import bzrlib.progress
from bzrlib.ui import UIFactory
import gtk

currentdialog = None


class PromptDialog(gtk.Dialog):
    """ Prompt the user for a yes/no answer. """
    def __init__(self, prompt):
        gtk.Dialog.__init__(self)

        label = gtk.Label(prompt)
        self.vbox.pack_start(label, padding=10)

        self.vbox.show_all()

        self.add_buttons(gtk.STOCK_YES, gtk.RESPONSE_YES, gtk.STOCK_NO,
                         gtk.RESPONSE_NO)


class GtkProgressBar(gtk.ProgressBar):
    data = {}
    _last_msg = ''

    def __init__(self, stack):
        super(GtkProgressBar, self).__init__()
        self.set_fraction(0.0)
        self._stack = stack

    def finished(self): 
        pass

    def clear(self):
        pass

    def tick(self):
        self.pulse()

    def update(self, msg=None, current=None, total=None):
        if msg is not None:
            self.set_text(msg)
            self._last_msg = msg
        else:
            msg = self._last_msg
            
        self.data.update({msg:[total,current]})
        nowtotal = 0
        nowcurrent = 0
        for it in self.data:
            if self.data[it][0] != None:
                nowtotal = nowtotal + self.data[it][0]
                nowcurrent = nowcurrent + self.data[it][1]
        while gtk.events_pending():
            gtk.main_iteration()
            
        self.set_fraction(1.0 * nowcurrent / nowtotal)
        
        while gtk.events_pending():
            gtk.main_iteration()


class GtkProgressBarStack(object):
    _nested = None
    def __init__(self):
        pass
    
    def get_nested(self):

        if self._nested == None:

            self._nested = GtkProgressBar(self)
            global currentdialog
            if currentdialog != None:
                currentdialog._statusbox.pack_start(self._nested)
            self._nested.show()

        while gtk.events_pending():
            gtk.main_iteration()

        return self._nested

    def remove(self, pb):
        pass


class PasswordDialog(gtk.Dialog):
    """ Prompt the user for a password. """
    def __init__(self, prompt):
        gtk.Dialog.__init__(self)

        label = gtk.Label(prompt)
        self.vbox.pack_start(label, padding=10)

        self.entry = gtk.Entry()
        self.entry.set_visibility(False)
        self.vbox.pack_end(self.entry, padding=10)

        self.vbox.show_all()

        self.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

    def _get_passwd(self):
        return self.entry.get_text()

    passwd = property(_get_passwd)


class GtkUIFactory(UIFactory):
    """A UI factory for GTK user interfaces."""

    def __init__(self,
                 stdout=None,
                 stderr=None):
        """Create a GtkUIFactory.

        """

        super(GtkUIFactory, self).__init__()
        self._progress_bar_stack = None

    def get_boolean(self, prompt):
        """GtkDialog with yes/no answers"""
        dialog = PromptDialog(prompt)
        response = dialog.run()
        dialog.destroy()
        return (response == gtk.RESPONSE_YES)

    def get_password(self, prompt='', **kwargs):
        """Prompt the user for a password.

        :param prompt: The prompt to present the user
        :param kwargs: Arguments which will be expanded into the prompt.
                       This lets front ends display different things if
                       they so choose.
        :return: The password string, return None if the user
                 canceled the request.
        """
        dialog = PasswordDialog(prompt % kwargs)
        response = dialog.run()
        passwd = dialog.passwd
        dialog.destroy()
        if response == gtk.RESPONSE_OK:
            return passwd
        else:
            return None

    def nested_progress_bar(self):
        """Return a nested progress bar.
        """
        if self._progress_bar_stack is None:
            self._progress_bar_stack = GtkProgressBarStack()
        return self._progress_bar_stack.get_nested()

    def set_progress_bar_vbox(self, vbox):
        """Change the vbox to put progress bars in.
        """
        self._progress_bar_stack = vbox

    def clear_term(self):
        """Prepare the terminal for output.

        It has no sense when talking about GTK."""
        pass