import os, re
import os.path

import gobject
import gtk
import pango
import time

from bzrlib.plugins.gtk import gettext
_i18n = gettext.gettext

from bzrlib import workingtree, inventory, errors
from bzrlib.errors import NotBranchError

#import gedit, gtksourceview2

class FileListTreeView(gtk.TreeView):
    
    def __init__(self, path = None, fileclickcb = None):
        gtk.TreeView.__init__(self)
        
        self.fileclickcb = fileclickcb
        
        self.set_property("headers-visible", True)
        self.set_property("rules-hint", True)
        self.get_selection().set_mode(gtk.SELECTION_SINGLE)
        self.connect("row-activated", self._row_activated)
        
        self._tvcolumn_filename = gtk.TreeViewColumn(_i18n('Filename'))
        self._tvcolumn_status = gtk.TreeViewColumn(_i18n('Status'))
        self._tvcolumn_size = gtk.TreeViewColumn(_i18n('Size'))
        self._tvcolumn_mtime = gtk.TreeViewColumn(_i18n('Last modified'))
        self.append_column(self._tvcolumn_filename)
        self.append_column(self._tvcolumn_status)
        self.append_column(self._tvcolumn_size)
        self.append_column(self._tvcolumn_mtime)
        
        
        # Set up the cells
        cellpb = gtk.CellRendererPixbuf()
        cell = gtk.CellRendererText()
        self._tvcolumn_filename.pack_start(cellpb, False)
        self._tvcolumn_filename.pack_start(cell, True)
        self._tvcolumn_filename.set_attributes(cellpb, stock_id=0)
        self._tvcolumn_filename.add_attribute(cell, 'text', 2)
        self._tvcolumn_status.pack_start(cell, True)
        self._tvcolumn_status.add_attribute(cell, 'text', 3)
        self._tvcolumn_size.pack_start(cell, True)
        self._tvcolumn_size.add_attribute(cell, 'text', 6)
        self._tvcolumn_mtime.pack_start(cell, True)
        self._tvcolumn_mtime.add_attribute(cell, 'text', 8)
        
        # Set up the properties of the TreeView
        self.set_headers_visible(True)
        self.set_headers_clickable(True)
        self.set_search_column(1)
        self._tvcolumn_filename.set_resizable(True)
        self._tvcolumn_status.set_resizable(True)
        self._tvcolumn_size.set_resizable(True)
        self._tvcolumn_mtime.set_resizable(True)
        self._tvcolumn_filename.set_sort_column_id(13)
        self._tvcolumn_status.set_sort_column_id(3)
        self._tvcolumn_size.set_sort_column_id(5)
        self._tvcolumn_mtime.set_sort_column_id(7)
        
        if path != None:
            self.set_path(path, True)
        
    def _row_activated(self, path, view_column, param):
        selpath = os.path.abspath(self.path + self.get_selected())
        
        if os.path.isdir(selpath):
            self.set_path(selpath,True)
        else:
            self.fileclickcb(selpath)
            #open file
        
    def get_selected(self):
        """ Get the selected filename. """
        treeselection = self.get_selection()
        (model, iter) = treeselection.get_selected()
        
        if iter is None:
            return None
        else:
            return model.get_value(iter, 2)
        
    def set_path(self, path, update = True):
        if path[-1] != '/':
            path = path + "/"
        try:
            self.wt, relpath = workingtree.WorkingTree.open_containing(path)
        except:
            self.clear_tree()
        else:
            self.path = path
            if update:
                self.draw_tree()
    
    def clear_tree(self):
        liststore = gtk.ListStore(gobject.TYPE_STRING)
        self.set_model(liststore)
    
    def update_file(self, file):
        """model = self.get_model()
        for row in model:
            if self.path + model.get_value(row.iter, 2) == file:"""
        self.draw_tree()
                
    def draw_tree(self):
        # Create ListStore
        # Model: [ icon, dir, name, status text, status, size (int), size (human), mtime (int), mtime (local), fileid ]
        wt = self.wt
        
        liststore = gtk.ListStore(gobject.TYPE_STRING,
                                  gobject.TYPE_BOOLEAN,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_INT,
                                  gobject.TYPE_STRING,
                                  gobject.TYPE_STRING)
        
        dirs = []
        files = []
        if wt.id2abspath(wt.get_root_id()) != self.path: #if we are in root, we can't go lower
            dirs.append("..")
        
        # Fill the appropriate lists
        dotted_files = True
        for item in os.listdir(self.path):
            if not dotted_files and item[0] == '.':
                continue
            if os.path.isdir(self.path + os.sep + item):
                dirs.append(item)
            else:
                files.append(item)
        
        branch = wt.branch
        tree2 = wt.branch.repository.revision_tree(branch.last_revision())
    
        delta = wt.changes_from(tree2, want_unchanged=True)
        
        # Add'em to the ListStore
        for item in dirs:
            try:
                statinfo = os.stat(self.path + os.sep + item)
            except OSError, e:
                if e.errno == 40:
                    continue
                else:
                    raise
            liststore.append([ gtk.STOCK_DIRECTORY,
                               True,
                               item,
                               '',
                               '',
                               "<DIR>",
                               "<DIR>",
                               statinfo.st_mtime,
                               self._format_date(statinfo.st_mtime),
                               ''])
            
        deltafiles = {}
        currpath = self.path.replace(wt.id2abspath(wt.get_root_id()),"")
        try:
            wt.lock_read()
            for rpath, id, kind in delta.unchanged:
                deltafiles.update({rpath : [id,'unchanged']})
            for rpath, id, kind, text_modified, meta_modified in delta.modified:
                deltafiles.update({rpath : [id,'modified']})
            for rpath, rpathnew, id, kind, text_modified, meta_modified in delta.renamed:
                deltafiles.update({rpath : [id,'renamed']})
            for rpath, id, kind in delta.added:
                deltafiles.update({rpath : [id,'added']})
            for rpath, id, kind in delta.removed:
                deltafiles.update({rpath : [id,'removed']})
            for rpath, file_class, kind, id, entry in wt.list_files():
                if not deltafiles.has_key(rpath) and file_class == 'I':
                    deltafiles.update({rpath : [id,'ignored']})
                elif not deltafiles.has_key(rpath):
                    deltafiles.update({rpath : [id,'unknow']})
        finally:
            wt.unlock()
        
        pathfiles = {}
        for item in files:
            filename = wt.relpath(self.path + os.sep + item)
            if deltafiles.has_key(filename):
                fileid = deltafiles[filename][0]
                st = deltafiles[filename][1]
            else:
                st = 'unknow'
                fileid = None
            status = st
            
        
            try:
                statinfo = os.stat(self.path + os.sep + item)
            except OSError, e:
                if e.errno == 40:
                    continue
                else:
                    raise
            liststore.append([gtk.STOCK_FILE,
                              False,
                              item,
                              st,
                              status,
                              str(statinfo.st_size), # NOTE: if int used there it will fail for large files (size expressed as long int)
                              self._format_size(statinfo.st_size),
                              statinfo.st_mtime,
                              self._format_date(statinfo.st_mtime),
                              fileid])

        # Create the columns and add them to the TreeView
        self.set_model(liststore)

        # Set up sorting
        liststore.set_sort_func(13, self._sort_filelist_callback, None)
        liststore.set_sort_column_id(13, gtk.SORT_ASCENDING)


        #self.show_all()
    def _sort_filelist_callback(self, model, iter1, iter2, data):
        """ The sort callback for the file list, return values:
        -1: iter1 < iter2
        0: iter1 = iter2
        1: iter1 > iter2
        """
        name1 = model.get_value(iter1, 2)
        name2 = model.get_value(iter2, 2)
        
        if model.get_value(iter1, 1):
            # item1 is a directory
            if not model.get_value(iter2, 1):
                # item2 isn't
                return -1
            else:
                # both of them are directories, we compare their names
                if name1 < name2:
                    return -1
                elif name1 == name2:
                    return 0
                else:
                    return 1
        else:
            # item1 is not a directory
            if model.get_value(iter2, 1):
                # item2 is
                return 1
            else:
                # both of them are files, compare them
                if name1 < name2:
                    return -1
                elif name1 == name2:
                    return 0
                else:
                    return 1
                
    def _format_date(self, timestamp):
        """ Format the time (given in secs) to a human readable format. """
        return time.ctime(timestamp)
    def _format_size(self, size):
        """ Format size to a human readable format. """
        if size < 1000:
            return "%d[B]" % (size,)
        size = size / 1000.0
        
        for metric in ["kB","MB","GB","TB"]:
            if size < 1000:
                break
            size = size / 1000.0
        return "%.1f[%s]" % (size,metric) 
    


if __name__ == '__main__':
    wnd = gtk.Window()
    tree = FileListTreeView("/home/javier/bzr/bzr.dev/")
    wnd.add(tree)
    wnd.show_all()
    #tree.set_path()
    gtk.main()